package model;

import java.util.ArrayList;

public class Pawn extends Piece {

	private boolean enPassant;
	public Pawn(Position p, int color) {
		super(p, color);
		enPassant = false;
	}

	public ArrayList<Position> getAvailableMoves(Board b) {

		// creates a dynamic list of available moves
		ArrayList<Position> moves = new ArrayList<>();
		Position arrival,enPassantEnemy;


		int y = -1;
		if(color == Piece.WHITE) { // black and white have an opposite direction behavior
			y = 1;
		}

		// forward
		arrival = new Position(p.getX(), p.getY() + y);
		if(b.isFree(arrival)) {
			moves.add(arrival);
			// forward twice, only if it's first move
			if(!hasBeenMoved){
				arrival = new Position(p.getX(), p.getY() + y + y);
				if(b.isFree(arrival)) { moves.add(arrival); }
			}
		}

		// forward + east, attack-only movement
		arrival = new Position(p.getX() + 1, p.getY() + y);
		if(b.isEnemy(arrival, p)) { moves.add(arrival); }

		// forward + west, attack-only movement
		arrival = new Position(p.getX() - 1, p.getY() + y);
		if(b.isEnemy(arrival, p)) { moves.add(arrival); }



		if(Piece.WHITE == color){
			if(p.getY() == 5){
				enPassantEnemy = new Position(p.getX()-1,p.getY());
				arrival = new Position(p.getX()-1,p.getY()+1);
				if(b.isEnemy(enPassantEnemy,p)&&
						b.isPawn(enPassantEnemy)&&
						((Pawn) b.getPiece(enPassantEnemy)).isEnPassant()&&
						b.isFree(arrival)){
						moves.add(arrival);
				}

				enPassantEnemy = new Position(p.getX()+1,p.getY());
				arrival = new Position(p.getX()+1,p.getY()+1);
				if(b.isEnemy(enPassantEnemy,p)&&
						b.isPawn(enPassantEnemy)&&
						((Pawn) b.getPiece(enPassantEnemy)).isEnPassant()&&
						b.isFree(arrival)){
					moves.add(arrival);
				}
			}

		}else {
			if(p.getY() == 4){

				enPassantEnemy = new Position(p.getX()-1,p.getY());
				arrival = new Position(p.getX()-1,p.getY()-1);
				if(b.isEnemy(enPassantEnemy,p)&&
						b.isPawn(enPassantEnemy)&&
						((Pawn) b.getPiece(enPassantEnemy)).isEnPassant()&&
						b.isFree(arrival)){
					moves.add(arrival);
				}

				enPassantEnemy = new Position(p.getX()+1,p.getY());
				arrival = new Position(p.getX()+1,p.getY()-1);
				if(b.isEnemy(enPassantEnemy,p)&&
						b.isPawn(enPassantEnemy)&&
						((Pawn) b.getPiece(enPassantEnemy)).isEnPassant()&&
						b.isFree(arrival)){
					moves.add(arrival);
				}
			}
		}


		return moves;


	}


	public boolean isEnPassant() {
		return enPassant;
	}

	public void setEnPassant(boolean enPassant) {
		this.enPassant = enPassant;
	}

	public Piece newInstanceCopy() {
		Pawn pawn = new Pawn(p, color);
		pawn.setEnPassant(enPassant);
		pawn.setHasBeenMoved(hasBeenMoved);
		return pawn;
	}

}

